#
# Copyright 2007 Fedora Unity
#
# Jonathan Steffan <jon a fedoraunity.org>
# Jeroen van Meeuwen <kanarip a fedoraunity.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os, sys, re

from optparse import OptionParser

# What it needs to do:
# - Given a checkbox to jigdofy whatever /installation media/ we compose, this stuff gets triggered
# - Create a subdirectory jigdo/ in self.cfg.destination_directory
# - Touch the jigdo file inside that directory
# - Use the installation tree
# - Generate the templates
# - Edit the .jigdo file to somehow use the repositories we used to compose the tree/media

# Problems:
# - When composing off Fedora/, jigdofy still requires you to specify the updates/ repo, and uses some packages from these
# updates/ repository as well.
# - We can't specify an online repo to use for comparing the ISO contents with the repository data, can we?
#
def set_defaults(cfg):
    cfg.defaults['jigdo_directory'] = ""
    cfg.defaults['jigdo_mirror_directory'] = ""

def check_options(cfg, cli_options):
    """Given a list of specified command line options in the OptionParser format
    this one checks if it's all OK"""

    # If the cli_options values are any different from the defaults...
    if not cli_options.jigdo_directory == cfg.defaults['jigdo_directory']:
        if os.access(cli_options.jigdo_directory, os.R_OK):
            # Directory already exists we're gonna bork!
            pass
        else:
            # No such directory maybe we should create it
            os.makedirs(cli_options.jigdo_directory)
            cfg.jigdo_directory = cli_options.jigdo_directory

    if not cli_options.jigdo_mirror_directory == cfg.defaults['jigdo_mirror_directory']:
        # Only bork if the directory does not exist, huh?
        if not os.access(cli_options.jigdo_mirror_directory, os.R_OK):
            # Bork! Bork!
            pass
        else:
            # Everything OK I presume...
            cfg.jigdo_mirror_directory = cli_options.jigdo_mirror_directory
            pass
    pass

def add_options(parser):
    """Adds a Jigdo Options group to the OptionParser instance you give it (parser),
    and adds the options for this module to that group"""
    jigdo_group = parser.add_option_group("Jigdo Options")
    jigdo_group.add_option( "--jigdo-directory",
                            dest    ="jigdo_directory",
                            action  ="store",
                            default ="",
                            help    =_("Generate Jigdo files and templates and put them in this directory"),
                            metavar ="[directory]")

    jigdo_group.add_option( "--mirror-directory",
                            dest    ="jigdo_mirror_directory",
                            action  ="store",
                            default ="",
                            help    =_("Use this directory as the base."),
                            metavar ="[directory]")

class RevisorJigdo:
    def __init__(self):
        parser = OptionParser()

        parser.add_option("--iso-directory", dest="iso_dir", action="store", default="", help="Directory to look for ISO images in.", metavar="[directory]")
        parser.add_option("--jigdo-directory", dest="jigdo_dir", action="store", default="", help="Directory to jigdo and template images in.", metavar="[directory]")

        parser.add_option("--repo-core", dest="repo_core", action="store", default="", help="Location of Core rpms.", metavar="[directory]")
        parser.add_option("--repo-updates", dest="repo_updates", action="store", default="", help="Location of Updates rpms.", metavar="[directory]")

        # Parse Options
        (options,args) = parser.parse_args()

    def the_rest_of_it(self):
        if options.iso_dir == "" or not os.access(options.iso_dir, os.R_OK):
            print "Please specify a valid directory for --iso-directory."
            sys.exit(1)

        if options.jigdo_dir == "" or not os.access(options.jigdo_dir, os.R_OK):
            print "Please specify a valid directory for --jigdo-directory."
            sys.exit(1)

        if options.repo_core == "" or not os.access(options.repo_core, os.R_OK):
            print "Please specify a valid directory for --repo-core."
            sys.exit(1)

        if options.repo_updates == "" or not os.access(options.repo_updates, os.R_OK):
            print "Please specify a valid directory for --repo-updates."
            sys.exit(1)


        def addISO((isos, ), path, files):
            """Find all ISO files in a given path."""
            for file in files:
                if os.path.exists(os.path.join(path, file)) and file.endswith('.iso'):
                    if os.path.exists(os.path.join(path, file)):
                        isos.append([path, file])

        print "\nScanning " + options.iso_dir + " for ISO images..."
        isos = []
        cdcommands32 = []
        cdcommands64 = []
        cdcommandsSource = []
        jigdo_dvd_command = "jigdo-file make-template --image=%s %s/ %s/ --label Core=%s --label Updates=%s --jigdo=%s --template=%s --no-servers-section"
        jigdo_cd_command = jigdo_dvd_command + " --force --merge=%s"

        os.path.walk(options.iso_dir, addISO, (isos,))

        # FIXME: Do something about x86_64 and i386

        isos.sort()

        for iso in isos:
            print "Building jigdo templates for " + iso[1] + "..."
            if re.compile('DVD').search(iso[1], 1):
                if re.compile('i386').search(iso[1], 1):
                    cdcommands32.append(jigdo_dvd_command % (os.path.join(iso[0], iso[1]), options.repo_core, options.repo_updates, options.repo_core, options.repo_updates, options.jigdo_dir + iso[1] + '.jigdo' , options.jigdo_dir + iso[1] + '.template'))
                elif re.compile('x86_64').search(iso[1], 1):
                    cdcommands64.append(jigdo_dvd_command % (os.path.join(iso[0], iso[1]), options.repo_core, options.repo_updates, options.repo_core, options.repo_updates, options.jigdo_dir + iso[1] + '.jigdo' , options.jigdo_dir + iso[1] + '.template'))
                elif re.compile('source').search(iso[1], 1):
                    print "Source building disabled... Skipping."
            elif re.compile('cd').search(iso[1], 1):
                if re.compile('i386').search(iso[1], 1):
                    cdcommands32.append(jigdo_cd_command % (os.path.join(iso[0], iso[1]), options.repo_core, options.repo_updates, options.repo_core, options.repo_updates, options.jigdo_dir + iso[1] + '.jigdo' , options.jigdo_dir + iso[1] + '.template', options.jigdo_dir + iso[1] + '.jigdo'))
                elif re.compile('x86_64').search(iso[1], 1):
                    cdcommands64.append(jigdo_cd_command % (os.path.join(iso[0], iso[1]), options.repo_core, options.repo_updates, options.repo_core, options.repo_updates, options.jigdo_dir + iso[1] + '.jigdo' , options.jigdo_dir + iso[1] + '.template', options.jigdo_dir + iso[1] + '.jigdo'))
                elif re.compile('source').search(iso[1], 1):
                    print "Source building disabled... Skipping."

        print "\n"
        print "32bit Images:\n"
        for command in cdcommands32:
            print command + ' \n'

        print "64bit Images:\n"
        for command in cdcommands64:
            print command + ' \n'

        example_servers = """Example Servers Section:
        [Server]
        Core=http://mirrors.kernel.org/fedora/core/6/i386/os/
        Updates=http://mirrors.kernel.org/fedora/core/6/x86_64/os/
        """

        print "For CD Sets, you need to make sure the .jigdo file exists: touch name.of.jigdo\n"

        print "[Server] and [Comments] sections need to be added. Also, 'ShortInfo=Terse Description' should be added to each [Image] section.\n"
        print example_servers

